<?php
/*
Based on WP-PageNavi plugin 

Plugin URI: http://wordpress.org/extend/plugins/wp-pagenavi/
Description: Adds a more advanced paging navigation to your WordPress blog.
Version: 2.70
Author: Lester 'GaMerZ' Chan
Author URI: http://lesterchan.net
Copyright 2009  Lester Chan  (email : lesterchan@gmail.com)
*/

### Function: Page Navigation: Boxed Style Paging
function wp_pagenavi_green($before = '', $after = '') {
	global $wp_query;

	$options = PageNavi_Core_green::$options->get();

	$posts_per_page = intval(get_query_var('posts_per_page'));
	$paged = intval(get_query_var('paged'));

	$request = $wp_query->request;
	$numposts = $wp_query->found_posts;
	$max_page = $wp_query->max_num_pages;

	if ( $max_page <= 1 && !intval($options['always_show']) )
		return;

	if ( empty($paged) )
		$paged = 1;

	$pages_to_show = intval($options['num_pages']);
	$larger_page_to_show = intval($options['num_larger_page_numbers']);
	$larger_page_multiple = intval($options['larger_page_numbers_multiple']);
	$pages_to_show_minus_1 = $pages_to_show - 1;
	$half_page_start = floor($pages_to_show_minus_1/2);
	$half_page_end = ceil($pages_to_show_minus_1/2);
	$start_page = $paged - $half_page_start;

	if ( $start_page <= 0 )
		$start_page = 1;

	$end_page = $paged + $half_page_end;

	if ( ($end_page - $start_page) != $pages_to_show_minus_1 )
		$end_page = $start_page + $pages_to_show_minus_1;

	if ( $end_page > $max_page ) {
		$start_page = $max_page - $pages_to_show_minus_1;
		$end_page = $max_page;
	}

	if ( $start_page <= 0 )
		$start_page = 1;

	$out = '';

    if ( $start_page >= 2 && $pages_to_show < $max_page ) {
        $out .= _wp_pagenavi_single_green(1, 'first', $options['first_text'], '%TOTAL_PAGES%');

        if ( !empty($options['dotleft_text']) )
            $out .= "<span class='extend left'>{$options['dotleft_text']}<em></em></span>";
    }

    $larger_pages_array = array();
    if ( $larger_page_multiple )
        for ( $i = $larger_page_multiple; $i <= $max_page; $i += $larger_page_multiple )
            $larger_pages_array[] = $i;

    $larger_page_start = 0;
    foreach ( $larger_pages_array as $larger_page ) {
        if ( $larger_page < $start_page && $larger_page_start < $larger_page_to_show ) {
            $out .= _wp_pagenavi_single_green($larger_page, 'smaller page', $options['page_text']);
            $larger_page_start++;
        }
    }

    if ( !empty($options['prev_text']) ) {
        $pl .= get_previous_posts_link($options['prev_text']);
        $pl = str_replace('</a>', '<em></em></a>', $pl);
        $out .= $pl;
    }

    for ( $i = $start_page; $i <= $end_page; $i++) {
        if ( $i == $paged && !empty($options['current_text']) ) {
            $current_page_text = str_replace('%PAGE_NUMBER%', number_format_i18n($i), $options['current_text']);
            $out .= "<span class='current'>$current_page_text<em></em></span>";
        } else {
            $out .= _wp_pagenavi_single_green($i, 'page', $options['page_text']);
        }
    }

    if ( !empty($options['next_text']) ) {
        $pl = get_next_posts_link($options['next_text'], $max_page);
        $pl = str_replace('</a>', '<em></em></a>', $pl);
        $out .= $pl;
    }

    $larger_page_end = 0;
    foreach ( $larger_pages_array as $larger_page ) {
        if ( $larger_page > $end_page && $larger_page_end < $larger_page_to_show ) {
            $out .= _wp_pagenavi_single_green($larger_page, 'larger page', $options['page_text']);
            $larger_page_end++;
        }
    }

    if ( $end_page < $max_page ) {
        if ( !empty($options['dotright_text']) )
            $out .= "<span class='extend right'>{$options['dotright_text']}<em></em></span>";

        $out .= _wp_pagenavi_single_green($max_page, 'last', $options['last_text'], '%TOTAL_PAGES%');
    }


	$out = $before . "<div class='wp-pagenavi'>".$out."<div class='clear'></div></div>" . $after;

	echo apply_filters('wp_pagenavi_green', $out);
}

function _wp_pagenavi_single_green($page, $class, $raw_text, $format = '%PAGE_NUMBER%') {
	if ( empty($raw_text) )
		return '';

	$text = str_replace($format, number_format_i18n($page), $raw_text);

	return "<a href='" . esc_url(get_pagenum_link($page)) . "' class='$class'>$text<em></em></a>";
}


### Function: Page Navigation: Drop Down Menu (Deprecated)


class PageNavi_Core_green {
	static $options;

	function init($options) {
		self::$options = $options;
	
		add_filter('previous_posts_link_attributes', array(__CLASS__, 'previous_posts_link_attributes'));
		add_filter('next_posts_link_attributes', array(__CLASS__, 'next_posts_link_attributes'));
	}

	function previous_posts_link_attributes() {
		return 'class="previouspostslink"';
	}

	function next_posts_link_attributes() {
		return 'class="nextpostslink"';
	}
}


function _pagenavi_init_green() {
	// Load scbFramework
	require dirname(__FILE__) . '/scb/load.php';

	load_plugin_textdomain('wp-pagenavi-green', false, 'wp-pagenavi-green');

	$options = new scbOptions('pagenavi_options', __FILE__, array(
		'current_text'  => '%PAGE_NUMBER%',
		'page_text'     => '%PAGE_NUMBER%',
		'first_text'    => '&lt;',
		'last_text'     => '&rt;',
		'prev_text'     => '&larr;',
		'next_text'     => '&rarr;',
		'dotleft_text'  => '...',
		'dotright_text' => '...',
		'num_pages' => 5,
		'num_larger_page_numbers' => 3,
		'larger_page_numbers_multiple' => 10,
		'always_show' => false,
	));

	PageNavi_Core_green::init($options);

}
_pagenavi_init_green();

