<?php
/**
 * Core functions file for the theme. Includes other key files.
 *
 * @package Suffusion
 * @subpackage Functions
 */

load_theme_textdomain("suf_theme", TEMPLATEPATH. "/translation");

$themename = "Suffusion";
$shortname = "suf";

// Option names
$color_scheme = $shortname."_color_scheme";
$sidebar_header = $shortname."_sidebar_header";
$show_shadows = $shortname."_show_shadows";
$body_style_setting = $shortname."_body_style_setting";
$body_font_style_setting = $shortname."_body_font_style_setting";
$header_style_setting = $shortname."_header_style_setting";

$suffusion_options = get_option('suffusion_options');
if ($suffusion_options[$color_scheme] === FALSE || $suffusion_options[$color_scheme] == null) {
	$theme_name = 'root';
}
else {
	$theme_name = $suffusion_options[$color_scheme];
}

//Essential for multi-selects. Do not delete!!! And do not move it below the next statement!!!
$spawned_options = array();
$page_array = null;
$category_array = null;

// Global variables
$SUFFUSION_COMMENT_TYPES = array('comment' => __('Comments', 'suf_theme'), 'trackback' => __('Trackbacks', 'suf_theme'), 'pingback' => __('Pingbacks', 'suf_theme'));

$sidebar_tabs = array(
	'archives' => array('title' => __('Archives', 'suf_theme')),
	'categories' => array('title' => __('Categories', 'suf_theme')),
	'links' => array('title' => __('Links', 'suf_theme')),
	'meta' => array('title' => __('Meta', 'suf_theme')),
	'pages' => array('title' => __('Pages', 'suf_theme')),
	'recent_comments' => array('title' => __('Recent Comments', 'suf_theme')),
	'recent_posts' => array('title' => __('Recent Posts', 'suf_theme')),
	'search' => array('title' => __('Search', 'suf_theme')),
	'tag_cloud' => array('title' => __('Tag Cloud', 'suf_theme')),
	'custom_tab_1' => array('title' => __('Custom Tab 1', 'suf_theme')),
	'custom_tab_2' => array('title' => __('Custom Tab 2', 'suf_theme')),
	'custom_tab_3' => array('title' => __('Custom Tab 3', 'suf_theme')),
	'custom_tab_4' => array('title' => __('Custom Tab 4', 'suf_theme')),
	'custom_tab_5' => array('title' => __('Custom Tab 5', 'suf_theme')),
	'custom_tab_6' => array('title' => __('Custom Tab 6', 'suf_theme')),
	'custom_tab_7' => array('title' => __('Custom Tab 7', 'suf_theme')),
	'custom_tab_8' => array('title' => __('Custom Tab 8', 'suf_theme')),
	'custom_tab_9' => array('title' => __('Custom Tab 9', 'suf_theme')),
	'custom_tab_10' => array('title' => __('Custom Tab 10', 'suf_theme')),
);

$suffusion_404_title =  __("Error 404 - Not Found", "suf_theme");
$suffusion_404_content = __("Sorry, the page that you are looking for does not exist.", "suf_theme");

$social_networks = array('twitter' => 'Twitter',
                'facebook' => 'Facebook',
                'technorati' => 'Technorati',
                'linkedin' => "LinkedIn",
                'flickr' => 'Flickr',
                'delicious' => 'Delicious',
                'digg' => 'Digg',
                'stumbleupon' => 'StumbleUpon',
                'reddit' => "Reddit");

include_once (TEMPLATEPATH . "/admin/theme-definitions.php");
include_once (TEMPLATEPATH . "/admin/theme-options.php");

function suffusion_add_admin() {
	global $themename, $shortname, $options, $spawned_options, $suffusion_options;
	if ($suffusion_options === FALSE || $suffusion_options == null || $suffusion_options == '') {
		$suffusion_options = array();
	}

	if ( $_GET['page'] == basename(__FILE__) ) {
		if ( 'save' == $_REQUEST['formaction'] ) {
			$form_category = $_REQUEST['formcategory'];
			$filtered_options = get_options_for_category($options, $form_category);

			foreach ($filtered_options as $value) {
				if(isset($_REQUEST[$value['id']])) {
					//update_option($value['id'], $_REQUEST[$value['id']]);
					$suffusion_options[$value['id']] = esc_html($_REQUEST[$value['id']]);
				}
				else {
					//delete_option( $value['id'] );
					unset($suffusion_options[$value['id']]);
				}
			}
			$filtered_spawned_options = get_spawned_options_for_category($options, $spawned_options, $form_category);
			$parents = array();
			foreach ($filtered_spawned_options as $value) {
				$parent_children = $parents[$value['parent']];
				if (!$parent_children) {
					$parent_children = array();
				}
				if(isset($_REQUEST[$value['id']])) {
					$parent_children[count($parent_children)] = substr($value['id'], strlen($value['parent']) + 1, strlen($value['id']) - strlen($value['parent']));
				}
				$parents[$value['parent']] = $parent_children;
			}
			foreach ($parents as $parent => $children) {
				if (is_array($children)) {
					$children_str = implode(',', $children);
					//update_option($parent, $children_str);
					$suffusion_options[$parent] = $children_str;
				}
				else {
					//delete_option($parent);
					unset($suffusion_options[$parent]);
				}
			}
			update_option('suffusion_options', $suffusion_options);
			header("Location: themes.php?page=functions.php&saved=true&category=$form_category"."&arr=".is_array($suffusion_options));
			die;
		}
		else if( 'reset' == $_REQUEST['formaction'] || 'reset_all' == $_REQUEST['formaction']) {
			$form_category = $_REQUEST['formcategory'];
			if ('reset_all' == $_REQUEST['formaction']) {
				$category = 'all';
			}
			else {
				$category = $form_category;
			}
			$filtered_options = get_options_for_category($options, $category);
			foreach ($filtered_options as $value) {
				//delete_option($value['id']);
				unset($suffusion_options[$value['id']]);
			}
			$filtered_spawned_options = get_spawned_options_for_category($options, $spawned_options, $category);
			foreach ($filtered_spawned_options as $value) {
				//delete_option( $value['id'] );
				unset($suffusion_options[$value['id']]);
			}
			update_option('suffusion_options', $suffusion_options);
			header("Location: themes.php?page=functions.php&".$_REQUEST['formaction']."=true&category=$form_category");
			die;
		}
		else if ('suf_up_migrate_302' == $_REQUEST['formaction']) {
			suffusion_migrate_from_v302();
		}
		else if ('suf_up_migrate_343' == $_REQUEST['formaction']) {
			suffusion_migrate_from_v343();
		}
		else if ('suf_export_options' == $_REQUEST['formaction']) {
			suffusion_export_settings();
		}
		else if ('suf_import_options' == $_REQUEST['formaction']) {
			suffusion_import_settings();
		}
	}
    $theme = current_theme_info();
    if (suffusion_update_available($theme)) {
        $update_available = "<img src='".get_stylesheet_directory_uri()."/admin/images/new.png' alt=''/>";
    }
	add_theme_page($themename." Theme Options", "".$themename." Theme Options $update_available", 'edit_themes', basename(__FILE__), 'suffusion_admin');

}

function suf_admin_header_style() {
    global $options;
    $option_structure = get_option_structure($options);
	if (isset($_REQUEST['category'])) {
		$category = $_REQUEST['category'];
        foreach ($option_structure as $l1) {
            if ($l1['parent'] == null) {
                foreach ($l1['children'] as $l2slug => $l2name) {
                    foreach ($option_structure[$l2slug]['children'] as $l3slug => $l3name) {
                        if ($category == $l3slug) {
                            $group = $l2slug;
                        }
                    }
                }
            }
        }
	}
	else {
		$category = 'welcome';
        $group = 'intro-pages';
	}
?>
	<link rel="stylesheet" href="<?php echo bloginfo('stylesheet_directory') . '/admin/admin.css?ver=3.4.7';?>" type="text/css" media="all"/>
	<link rel="stylesheet" href="<?php echo bloginfo('stylesheet_directory') . '/admin/jquery-ui/css/jquery-ui-1.7.2.custom.css?ver=3.4.2';?>" type="text/css" media="all"/>
	<script type="text/javascript" src="<?php echo bloginfo('stylesheet_directory') . '/admin/jscolor/jscolor.js';?>"></script>
	<script type="text/javascript">
		/* <![CDATA[ */
		$j = jQuery.noConflict();

		$j(document).ready(function() {
			var selected_category = '<?php echo $category; ?>';
			var selected_group = '<?php echo $group; ?>';
			var current_group = selected_group;
			$j('div.suf-options').hide();
			$j('div.main-content').hide();
			$j('div.suf-options-' + selected_group).show();
			$j('.htab-' + selected_group).addClass('current-tab');
			$j('#' + selected_category).show();
			$j('div.suf-options ul.tabs li.' + selected_category + ' a').addClass('current-tab');
			$j('ul.htabs li').click(function(){
				var thisClass = this.className.substring(5, this.className.indexOf(" "));
				$j('div.suf-options').hide();
				$j('div.suf-options-' + thisClass).show();
				$j('ul.htabs li').removeClass('current-tab');
				$j(this).addClass('current-tab');
				if (current_group != thisClass) {
    				$j('div.suf-options ul.tabs li a').removeClass('current-tab');
        			$j('ul.tabs').find('li:first a').addClass('current-tab');
    	    		$j('div.suf-options-' + thisClass + ' div.main-content').hide();
			    	$j('div.suf-options-' + thisClass).find('.main-content:first').show();
			    	current_group = thisClass;
				}
			});

			$j('ul.tabs li a').click(function(){
				var thisClass = this.className.substring(0, this.className.indexOf(" "));
				$j('div.suf-options div.main-content').hide();
				$j('#' + thisClass).show();
				$j('div.suf-options ul.tabs li a').removeClass('current-tab');
				$j(this).addClass('current-tab');
			});

			$j('input.suf-multi-select-button').click(function() {
			    var thisAction = this.className.substring(0, this.className.indexOf(" "));
			    var thisName = this.name.substring(0, this.name.indexOf("-"));
			    if (thisAction == 'button-all') {
			        $j('input[type=checkbox].suf-options-checkbox-' + thisName).attr('checked', true);
			    }
			    else if (thisAction == 'button-none') {
			        $j('input[type=checkbox].suf-options-checkbox-' + thisName).attr('checked', false);
			    }
			});

			$j('.suf-grouping').each(function() {
				var thisName = this.className.substring(0, this.className.indexOf(" "));
				thisName = thisName.substring(13);
				$j('.' + thisName + '-grouping .' + thisName + '-rhs').append(this);
				//$j(this).prependTo(this.parentNode.parentNode);
			});

			$j('.suf-background-options input[type=text]').change(function(event) {
				var thisName = event.currentTarget.id;
				thisName = thisName.substring(0, thisName.indexOf('-'));
				$j("#" + thisName).val('color=' + $j("#" + thisName + "-bgcolor").val() + ';' +
									   'colortype=' + $j("input[name=" + thisName + "-colortype]:checked").val() + ';' +
									   'image=' + $j("#" + thisName + "-bgimg").val() + ';' +
									   'position=' + $j("#" + thisName + "-position").val() + ';' +
									   'repeat=' + $j("#" + thisName + "-repeat").val() + ';' +
									   'trans=' + $j("#" + thisName + "-trans").val() + ';'
						);
			});

			$j('.suf-background-options input[type=radio]').change(function(event) {
				var thisName = event.currentTarget.name;
				thisName = thisName.substring(0, thisName.indexOf('-'));
				$j("#" + thisName).val('color=' + $j("#" + thisName + "-bgcolor").val() + ';' +
									   'colortype=' + $j("input[name=" + thisName + "-colortype]:checked").val() + ';' +
									   'image=' + $j("#" + thisName + "-bgimg").val() + ';' +
									   'position=' + $j("#" + thisName + "-position").val() + ';' +
									   'repeat=' + $j("#" + thisName + "-repeat").val() + ';' +
									   'trans=' + $j("#" + thisName + "-trans").val() + ';'
						);
			});

			$j('.suf-background-options select').change(function(event) {
				var thisName = event.currentTarget.id;
				thisName = thisName.substring(0, thisName.indexOf('-'));
				$j("#" + thisName).val('color=' + $j("#" + thisName + "-bgcolor").val() + ';' +
									   'colortype=' + $j("input[name=" + thisName + "-colortype]:checked").val() + ';' +
									   'image=' + $j("#" + thisName + "-bgimg").val() + ';' +
									   'position=' + $j("#" + thisName + "-position").val() + ';' +
									   'repeat=' + $j("#" + thisName + "-repeat").val() + ';' +
									   'trans=' + $j("#" + thisName + "-trans").val() + ';'
						);
			});
		});
		function submit_form(field, form) {
			form.elements['formaction'].value = field.name;
			if (field.name == 'reset_all') {
				if (confirm("This will reset ALL your configurations to the original values!!! Are you sure you want to continue?")) {
					form.submit();
				}
			}
			else if (field.name == 'reset') {
				if (confirm("This will reset all options on this page to the original values!!! Are you sure you want to continue?")) {
					form.submit();
				}
			}
			else {
				form.submit();
			}
		}
		/* ]]> */
	</script>
<?php
}
add_action('admin_head', 'suf_admin_header_style');

function suffusion_admin_script_loader() {
	wp_enqueue_script('jquery-ui-sortable');
	wp_enqueue_script('jquery-ui-custom', get_bloginfo('template_directory') . '/admin/jquery-ui/jquery-ui-custom.js', array('jquery-ui-core'));
}
add_action('admin_print_scripts', 'suffusion_admin_script_loader');

function suffusion_admin() {
	global $themename, $shortname, $options, $spawned_options, $theme_name, $suf_theme_definitions;
	if ( $_REQUEST['saved'] ) echo '<div id="message" class="updated fade"><p><strong>'.$themename.' settings saved for this page.</strong></p></div>';
	if ( $_REQUEST['reset'] ) echo '<div id="message" class="updated fade"><p><strong>'.$themename.' settings reset for this page.</strong></p></div>';
	if ( $_REQUEST['reset_all'] ) echo '<div id="message" class="updated fade"><p><strong>'.$themename.' settings reset for all pages.</strong></p></div>';
	if ( $_REQUEST['migrated'] ) echo '<div id="message" class="updated fade"><p><strong>'.$themename.' options migrated successfully.</strong></p></div>';
	if ( $_REQUEST['imported'] ) echo '<div id="message" class="updated fade"><p><strong>'.$themename.' options imported successfully.</strong></p></div>';
?>
<div class="wrap">
<div class="suf-tabbed-options">
<h2 class='suf-header-1'>Settings for <?php echo $themename; ?></h2>
<?php
    $theme = current_theme_info();
    theme_update_available($theme);
echo "<!-- all suffusion options ";print_r(get_option('suffusion_options')); echo "-->";
	$option_structure = get_option_structure($options);
	get_options_html($option_structure);
?>
	</div><!-- suf-options -->
	</div><!-- wrap -->
	<?php
}

add_action('admin_menu', 'suffusion_add_admin');

// Begin legacy comments support
add_filter( 'comments_template', 'legacy_comments' );
function legacy_comments( $file ) {
	if ( !function_exists('wp_list_comments') )
		$file = TEMPLATEPATH . '/legacy-comments.php';
	return $file;
}
//End legacy comments support

function suffusion_migrate_from_v302() {
	global $spawned_options, $suffusion_options;
	if ($suffusion_options === FALSE || $suffusion_options == null || $suffusion_options == '') {
		$suffusion_options = array();
	}

	$form_category = $_REQUEST['formcategory'];
	$parent_spawn_groups = array();
	foreach ($spawned_options as $spawn) {
		$parent = $spawn['parent'];
		if (!$parent_spawn_groups[$parent]) {
			$parent_spawn_groups[$parent] = array();
		}
		$parent_spawn = $parent_spawn_groups[$parent];
		$parent_spawn[count($parent_spawn)] = $spawn['id'];
		$parent_spawn_groups[$parent] = $parent_spawn;
	}

	$new_values = array();
	foreach ($parent_spawn_groups as $parent => $spawn_group) {
		$new_values[$parent] = array();
	}

	foreach ($parent_spawn_groups as $parent => $spawn_group) {
		$selected_children = $new_values[$parent];
		foreach ($spawn_group as $idx => $spawn) {
			if (get_option($spawn)) {
				$selected_children[count($selected_children)] = substr($spawn, strlen($parent) + 1, strlen($spawn) - strlen($parent));
			}
		}
		$new_values[$parent] = $selected_children;
	}

	foreach ($new_values as $parent => $spawn_group) {
		$consolidated = implode(',', $spawn_group);
		//update_option($parent, $consolidated);
		$suffusion_options[$parent] = $consolidated;
	}
	update_option('suffusion_options', $suffusion_options);

	$meta_fields = array('suf_alt_page_title' => 'text');
	foreach ($meta_fields as $meta_field => $type) {
		$pages = get_pages();
		if ($pages && is_array($pages)) {
			foreach ($pages as $page) {
				$page_id = $page->ID;
				if ($page != null) {
					if ($type == 'checkbox') {
						$data = 'on';
					}
					else if ($type == 'text') {
						$data = get_option($meta_field.'_'.$page_id);
					}
					if (get_post_meta($page_id, $meta_field) == '') {
						add_post_meta($page_id, $meta_field, $data, true);
					}
					else if ($data != get_post_meta($page_id, $meta_field, true)) {
						update_post_meta($page_id, $meta_field, $data);
					}
					else if ($data == '') {
						delete_post_meta($page_id, $meta_field, get_post_meta( $page_id, $meta_field, true));
					}
				}
			}
		}
	}
	header("Location: themes.php?page=functions.php&migrated=true&category=$form_category"."&data=");
}

function suf_get_formatted_page_array($prefix) {
	global $spawned_options, $page_array;
	$ret = array();
	$pages = get_pages('sort_column=menu_order');
    if ($pages != null) {
        foreach ($pages as $page) {
            if ($page_array == null) {
                $ret[$page->ID] = array ("title" => $page->post_title, "depth" => count(get_post_ancestors($page)));
            }
            $spawned_options[count($spawned_options)] = array(  "id" => $prefix."_".$page->ID,
                "type" => "checkbox",
                "parent" => $prefix, "std" => "false");
        }
    }
	if ($page_array == null) {
		$page_array = $ret;
		return $ret;
	}
	else {
		return $page_array;
	}
}

function suf_get_formatted_category_array($prefix, $spawn = true) {
	global $spawned_options, $category_array;
	$ret = array();
	$args = array("type" => "post",
		"orderby" => "name",
		"hide_empty" => false,
	);
	$categories = get_categories($args);
	if ($categories == null) { $categories = array(); }
	foreach ($categories as $category) {
		if ($category_array == null) {
			$ret[$category->cat_ID] = array("title" => $category->cat_name);
		}
		if ($spawn) {
			$spawned_options[count($spawned_options)] = array(  "id" => $prefix."_".$category->cat_ID,
														        "type" => "checkbox",
														        "parent" => $prefix,
																"std" => "false");
		}
	}
	if ($category_array == null) {
		$category_array = $ret;
		return $category_array;
	}
	else {
		return $category_array;
	}
}

function suf_get_allowed_categories($prefix) {
	global $suffusion_options;
//	$selected = get_option($prefix);
	$selected = $suffusion_options[$prefix];
	if ($selected && trim($selected) != '') { $selected_categories = explode(',', $selected); } else { $selected_categories = array(); }
	$allowed = array();
	if ($selected_categories && is_array($selected_categories)) {
		foreach ($selected_categories as $category) {
			$allowed[count($allowed)] = get_category($category);
		}
	}
	return $allowed;
}

if (class_exists("WP_Widget")) {
	include_once (TEMPLATEPATH . '/suffusion-widgets.php');
}
include_once (TEMPLATEPATH . '/suffusion-classes.php');

//if (current_user_can("manage_options")) {
	add_action('admin_menu', 'suffusion_add_page_fields');
//}

function suffusion_add_page_fields() {
	if (function_exists('add_meta_box')) {
		add_meta_box('suffusion-page-box', 'Additional Options for Suffusion', 'suffusion_page_extras', 'page', 'normal', 'high');
		add_meta_box('suffusion-post-box', 'Additional Options for Suffusion', 'suffusion_post_extras', 'post', 'normal', 'high');
	}
}

function suffusion_page_extras() {
	global $post;
?>
	<p>
		<label for="suf_alt_page_title"><?php _e("Page Title in Dropdown Menu", "suf_theme"); ?></label><br/>
		<input type="text" id="suf_alt_page_title" name="suf_alt_page_title"
		value="<?php echo get_post_meta($post->ID, "suf_alt_page_title", true); ?>" /> <?php _e("This text will be shown in the drop-down menus in the navigation bar. If left blank, the title of the page is used.", 'suf_theme'); ?>
	</p>
	<p>
		<label for="suf_nav_unlinked"><?php _e("Do not link to this page in the navigation bars", "suf_theme"); ?></label><br/>
		<input type="checkbox" id="suf_nav_unlinked" name="suf_nav_unlinked"
			<?php if (get_post_meta($post->ID, 'suf_nav_unlinked', true)) { echo " checked='checked' ";} ?> />
			<?php _e('If this box is checked, clicking on this page in the navigation bar will not take you anywhere.', 'suf_theme'); ?>
	</p>
	<p>
		<label for="thumbnail"><?php _e("Thumbnail", "suf_theme"); ?></label><br/>
		<input type="text" id="thumbnail" name="thumbnail"
			value="<?php echo get_post_meta($post->ID, "thumbnail", true); ?>" /> <?php _e("Enter the full URL of the thumbnail image that you would like to use, including http://", "suf_theme"); ?>
	</p>
	<p>
		<label for="featured_image"><?php _e("Featured Image", "suf_theme"); ?></label><br/>
		<input type="text" id="featured_image" name="featured_image"
			value="<?php echo get_post_meta($post->ID, "featured_image", true); ?>" /> <?php _e("Enter the full URL of the featured image that you would like to use, including http://", "suf_theme"); ?>
	</p>
	<p>
		<label for="meta_description"><?php _e("Meta Description", "suf_theme"); ?></label><br/>
		<textarea id="meta_description" name="meta_description" cols='80' rows='5'><?php echo get_post_meta($post->ID, "meta_description", true); ?></textarea>
	</p>
	<p>
		<label for="meta_keywords"><?php _e("Meta Keywords", "suf_theme"); ?></label><br/>
		<input type="text" id="meta_keywords" name="meta_keywords" style='width: 500px;'
			value="<?php echo get_post_meta($post->ID, "meta_keywords", true); ?>" /> <?php _e("Enter a comma-separated list of keywords for this post. This list will be included in the meta tags for this post.", "suf_theme"); ?>
	</p>
	<input type='hidden' id='suffusion_post_meta' name='suffusion_post_meta' value='suffusion_post_meta'/>
<?php
}

function suffusion_post_extras() {
	global $post;
?>
	<p>
		<label for="thumbnail"><?php _e("Thumbnail", "suf_theme"); ?></label><br/>
		<input type="text" id="thumbnail" name="thumbnail"
			value="<?php echo get_post_meta($post->ID, "thumbnail", true); ?>" /> <?php _e("Enter the full URL of the thumbnail image that you would like to use, including http://", "suf_theme"); ?>
	</p>
	<p>
		<label for="featured_image"><?php _e("Featured Image", "suf_theme"); ?></label><br/>
		<input type="text" id="featured_image" name="featured_image"
			value="<?php echo get_post_meta($post->ID, "featured_image", true); ?>" /> <?php _e("Enter the full URL of the featured image that you would like to use, including http://", "suf_theme"); ?>
	</p>
	<p>
		<label for="suf_magazine_headline"><?php _e("Make this post a headline", "suf_theme"); ?></label><br/>
		<input type="checkbox" id="suf_magazine_headline" name="suf_magazine_headline"
			<?php if (get_post_meta($post->ID, 'suf_magazine_headline', true)) { echo " checked='checked' ";} ?> />
			<?php _e('If this box is checked, this post will show up as a headline in the magazine template.', 'suf_theme'); ?>
	</p>
	<p>
		<label for="suf_magazine_excerpt"><?php _e("Make this post an excerpt in the magazine layout", "suf_theme"); ?></label><br/>
		<input type="checkbox" id="suf_magazine_excerpt" name="suf_magazine_excerpt"
			<?php if (get_post_meta($post->ID, 'suf_magazine_excerpt', true)) { echo " checked='checked' ";} ?> />
			<?php _e('If this box is checked, this post will show up as an excerpt in the magazine template.', 'suf_theme'); ?>
	</p>
	<p>
		<label for="meta_description"><?php _e("Meta Description", "suf_theme"); ?></label><br/>
		<textarea id="meta_description" name="meta_description" cols='80' rows='5'><?php echo get_post_meta($post->ID, "meta_description", true); ?></textarea>
	</p>
	<p>
		<label for="meta_keywords"><?php _e("Meta Keywords", "suf_theme"); ?></label><br/>
		<input type="text" id="meta_keywords" name="meta_keywords" style='width: 500px;'
			value="<?php echo get_post_meta($post->ID, "meta_keywords", true); ?>" /> <?php _e("Enter a comma-separated list of keywords for this post. This list will be included in the meta tags for this post.", "suf_theme"); ?>
	</p>
	<input type='hidden' id='suffusion_post_meta' name='suffusion_post_meta' value='suffusion_post_meta'/>
<?php
}

function suffusion_save_post_fields($post_id) {
	$suffusion_post_fields = array('thumbnail', 'featured_image', 'suf_magazine_headline', 'suf_magazine_excerpt', 'suf_alt_page_title', 'meta_description', 'meta_keywords', 'suf_nav_unlinked');
    if (isset($_POST['suffusion_post_meta'])) {
        foreach ($suffusion_post_fields as $post_field) {
            $data = stripslashes($_POST[$post_field]);
            if (get_post_meta($post_id, $post_field) == '') {
                add_post_meta($post_id, $post_field, $data, true);
            }
            else if ($data != get_post_meta($post_id, $post_field, true)) {
                update_post_meta($post_id, $post_field, $data);
            }
            else if ($data == '') {
                delete_post_meta($post_id, $post_field, get_post_meta($post_id, $post_field, true));
            }
        }
    }
}

add_action("save_post", "suffusion_save_post_fields");
add_action("publish_post", "suffusion_save_post_fields");

function suffusion_export_settings() {
	global $options, $suffusion_options;
	$export = array();
	foreach ($options as $value) {
		if ($value['export'] == 'ne' || !isset($value['id']) || $value['type'] == 'button') {
			continue;
		}
		if (!isset($suffusion_options[$value['id']])) {
			$export[$value['id']] = $value['std'];
		}
		else {
			$export[$value['id']] = $suffusion_options[$value['id']];
		}
	}
	header('Content-Type: text/plain');
	header('Content-Disposition: attachment; filename="suffusion-options.php"');
	echo "<?php \n";
	echo "/* Suffusion settings exported on ".date('Y-m-d H:i')." */ \n";
	echo '$suffusion_exported_options = ';
	var_export($export);
	echo ";\n ?>";
	die;
}

function suffusion_import_settings() {
	global $suffusion_options;
	if ($suffusion_options === FALSE || $suffusion_options == null || $suffusion_options == '') {
		$suffusion_options = array();
	}

	$form_category = $_REQUEST['formcategory'];
	if (file_exists(TEMPLATEPATH."/admin/import/suffusion-options.php")) {
		include (TEMPLATEPATH."/admin/import/suffusion-options.php");
		foreach ($suffusion_exported_options as $option => $option_value) {
			//update_option($option, $option_value);
			$suffusion_options[$option] = $option_value;
		}
		update_option('suffusion_options', $suffusion_options);
		header("Location: themes.php?page=functions.php&imported=true&category=$form_category"."&data=");
	}
	else {
		header("Location: themes.php?page=functions.php&imported=false&category=$form_category"."&data=");
	}
}

function suffusion_migrate_from_v343() {
	global $options;
	$suffusion_options = array();

	$form_category = $_REQUEST['formcategory'];
	foreach ($options as $value) {
		if (get_option($value['id']) === FALSE) {
			unset($suffusion_options[$value['id']]);
		}
		else {
			$suffusion_options[$value['id']] = get_option($value['id']);
		}
	}
	update_option('suffusion_options', $suffusion_options);
	header("Location: themes.php?page=functions.php&migrated=true&category=$form_category"."&data=");
}

add_action('wp_head', 'suf_add_header_contents');
function suf_add_header_contents() {
	suf_create_openid_links();
	suf_create_additional_feeds();
}

add_action('wp_footer', 'suf_add_footer_contents');
function suf_add_footer_contents() {
	suf_create_analytics_contents();
}

// OpenID stuff...
function suf_create_openid_links() {
	global $suffusion_options;
	if ($suffusion_options['suf_openid_enabled'] == "enabled") {
		echo "<!-- Start OpenID settings -->\n";
		echo "<link rel=\"openid.server\" href=\"".$suffusion_options['suf_openid_server']."\" />\n";
		echo "<link rel=\"openid.delegate\" href=\"".$suffusion_options['suf_openid_delegate']."\" />\n";
		echo "<!-- End OpenID settings -->\n";
	}
}
// ... End OpenID stuff

// Analytics ...
function suf_create_analytics_contents() {
	global $suffusion_options;
	if ($suffusion_options['suf_analytics_enabled'] == "enabled") {
		if (trim($suffusion_options['suf_custom_analytics_code']) != "") {
			echo "<!-- Start Google Analytics -->\n";
			echo stripslashes($suffusion_options['suf_custom_analytics_code'])."\n";
			echo "<!-- End Google Analytics -->\n";
		}
	}
}
// ... End Analytics

// Additional Feeds ...
function suf_create_additional_feeds() {
	global $suffusion_options;
	echo "<!-- Start Additional Feeds -->\n";
	if (trim($suffusion_options['suf_custom_rss_feed_1']) != "") {
		echo "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"".$suffusion_options['suf_custom_rss_title_1']."\" href=\"".$suffusion_options['suf_custom_rss_feed_1']."\" />\n";
	}
	if (trim($suffusion_options['suf_custom_rss_feed_2']) != "") {
		echo "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"".$suffusion_options['suf_custom_rss_title_2']."\" href=\"".$suffusion_options['suf_custom_rss_feed_2']."\" />\n";
	}
	if (trim($suffusion_options['suf_custom_rss_feed_3']) != "") {
		echo "<link rel=\"alternate\" type=\"application/rss+xml\" title=\"".$suffusion_options['suf_custom_rss_title_3']."\" href=\"".$suffusion_options['suf_custom_rss_feed_3']."\" />\n";
	}
	if (trim($suffusion_options['suf_custom_atom_feed_1']) != "") {
		echo "<link rel=\"alternate\" type=\"application/atom+xml\" title=\"".$suffusion_options['suf_custom_atom_title_1']."\" href=\"".$suffusion_options['suf_custom_atom_feed_1']."\" />\n";
	}
	if (trim($suffusion_options['suf_custom_atom_feed_2']) != "") {
		echo "<link rel=\"alternate\" type=\"application/atom+xml\" title=\"".$suffusion_options['suf_custom_atom_title_2']."\" href=\"".$suffusion_options['suf_custom_atom_feed_2']."\" />\n";
	}
	if (trim($suffusion_options['suf_custom_atom_feed_3']) != "") {
		echo "<link rel=\"alternate\" type=\"application/atom+xml\" title=\"".$suffusion_options['suf_custom_atom_title_3']."\" href=\"".$suffusion_options['suf_custom_atom_feed_3']."\" />\n";
	}
	echo "<!-- End Additional Feeds -->\n";
}
// ... End Additional Feeds

function get_excluded_pages($prefix) {
    global $$prefix;
    $inclusions = $$prefix;
	$all_pages = get_pages('sort_column=menu_order');
	if ($all_pages == null) {
		$all_pages = array();
	}

    if ($inclusions && trim($inclusions) != '') {
        $include = explode(',', $inclusions);
        $translations = wpml_lang_object_ids($include, 'post');
        foreach ($translations as $translation) {
            $include[count($include)] = $translation;
        }
    }
    else {
        $include = array();
    }

	// First we figure out which pages have to be excluded
	$exclude = array();
	foreach ($all_pages as $page) {
		if (!in_array($page->ID, $include)) {
            $exclude[count($exclude)] = $page->ID;
        }
	}
	// Now we need to figure out if these excluded pages are ancestors of any pages on the list. If so, we remove the descendants
	foreach ($all_pages as $page) {
		$ancestors = get_post_ancestors($page);
		foreach ($ancestors as $ancestor) {
			if (in_array($ancestor, $exclude)) {
				$exclude[count($exclude)] = $page->ID;
			}
		}
	}

	$exclusion_list = implode(",", $exclude);
	return $exclusion_list;
}

function suffusion_get_page_structure($id, $title, $link, $depth, $exclusion_list) {
	$page_structure = new Suf_Navigation_Structure($id, $title, $link, $depth, 'page');
	$page_structure->children = array();
	$child_pages = get_pages('sort_column=menu_order&parent='.$id.'&child_of='.$id.'&exclude='.$exclusion_list);
	if ($child_pages == null) { $child_pages = array(); }
	$child_depth = $depth + 1;
	foreach ($child_pages as $child_page) {
		$link_disabled = get_post_meta($child_page->ID, 'suf_nav_unlinked', true);
		if ($link_disabled) {
			$child_page_link = "";
		}
		else {
			$child_page_link = get_page_link($child_page->ID);
		}
		$child = suffusion_get_page_structure($child_page->ID, $child_page->post_title, $child_page_link, $child_depth, $exclusion_list);
		$page_structure->children[count($page_structure->children)] = $child;
	}
	return $page_structure;
}

function suffusion_get_home_link_html($position) {
	global $suffusion_options;
	$retStr = "";
	$suf_show_home = "";
	$option_name = $position == 'top' ? 'suf_navt_show_home' : 'suf_show_home';
	if ($suffusion_options[$option_name] === FALSE || $suffusion_options[$option_name] == null) {
		$suf_show_home = "none";
	}
	else {
		$suf_show_home = $suffusion_options[$option_name];
	}

    $show_on_front = get_option('show_on_front');
    $page_for_posts = get_option('page_for_posts');
    if (is_front_page()) {
        $class = " class='highlighted' ";
    }
    else if (is_home() && $show_on_front == 'posts') {
        $class = " class='highlighted' ";
    }

	$option_name = $position == 'top' ? 'suf_navt_home_text' : 'suf_home_text';
	$home_link = get_option("home");
	if (function_exists('icl_get_home_url')) {
		$home_link = icl_get_home_url();
	}
	if ($suf_show_home == "text") {
		if ($suffusion_options[$option_name] === FALSE || $suffusion_options[$option_name] == null) {
			$suf_home_text = "Home";
		}
		else if (trim($suffusion_options[$option_name]) == "") {
			$suf_home_text = "Home";
		}
		else {
			$suf_home_text = trim($suffusion_options[$option_name]);
		}
		$retStr .= "\n\t\t\t\t\t"."<li><a $class href='".$home_link."'>".$suf_home_text."</a></li>";
	}
	else if ($suf_show_home == "icon") {
		$retStr .= "\n\t\t\t\t\t"."<li><a $class href='".$home_link."'><img src='".get_bloginfo("template_directory")."/images/home-light.png' alt='Home' class='home-icon'/></a></li>";
	}
	return $retStr;
}

/**
 * This method takes a Suffusion_Nav_Structure object and parses it out to build the navigation menu.
 * The menu is based on Stu Nicholls' famous CSS-only navigation menu: http://www.cssplay.co.uk/menus/final_drop.html. I have made minor modifications to it.
 */
function suffusion_parse_nav_structure($nav_structure, $position = 'main') {
	global $post;
	$retStr = "";
	if ($nav_structure->depth == 0) {
		$home_link = suffusion_get_home_link_html($position);
		if (($nav_structure->children != null && is_array($nav_structure->children) && count($nav_structure->children) > 0) || $home_link) {
			$retStr .= "\n\t\t\t\t\t"."<ul class='sf-menu'>";
			$retStr .= $home_link;
			foreach ($nav_structure->children as $child_page) {
				$retStr .= suffusion_parse_nav_structure($child_page, $position);
			}
			$retStr .= "\n\t\t\t\t\t"."</ul>";
		}
	}
	else if ($nav_structure->depth == 1) {
        $show_on_front = get_option('show_on_front');
        $page_for_posts = get_option('page_for_posts');
		if (is_array($post->ancestors)) {
			$ancestors = $post->ancestors;
		}
		else {
			$ancestors = array();
		}

		$href = trim($nav_structure->link) == "" ? " href='#' onclick='return false;' " : " href='$nav_structure->link' ";
		if (is_page() && ($post->ID == $nav_structure->ID || in_array($nav_structure->ID, $ancestors)) && $nav_structure->type == 'page' ||
                (($nav_structure->ID == $page_for_posts) && $show_on_front == 'page' && is_home())) {
			$retStr .= "\n\t\t\t\t\t"."<li><a $href class='highlighted'>".$nav_structure->title."</a>";
		}
		else if (is_category() && (single_cat_title('', false) == $nav_structure->title)&& $nav_structure->type == 'cat') {
			$retStr .= "\n\t\t\t\t\t"."<li><a $href class='highlighted'>".$nav_structure->title."</a>";
		}
		else {
			$target = $nav_structure->target ? " target='".$nav_structure->target."' " : ' ';
			$retStr .= "\n\t\t\t\t\t"."<li><a $href $target>".$nav_structure->title."</a>";
		}
		if ($nav_structure->children != null && count($nav_structure->children) != 0) {
			$retStr .= "\n\t\t\t"."<ul>";
			foreach ($nav_structure->children as $child_page) {
				$target = $child_page->target ? " target='".$child_page->target."' " : ' ';
				$child_href = trim($child_page->link) == "" ? " href='#' onclick='return false;' " : " href='$child_page->link' ";
				//$retStr .= "\n\t\t\t\t"."<li><a href='$child_page->link' $target";
				$retStr .= "\n\t\t\t\t"."<li><a $child_href $target";
				$retStr .= suffusion_parse_nav_structure($child_page, $position);
			}
			$retStr .= "\n\t\t\t\t"."</ul>";
		}
        $retStr .= "\n\t\t\t\t"."</li>";
	}
	else {
		if ($nav_structure->children != null && count($nav_structure->children) != 0) {
			$retStr .= " class='drop' title=\"$nav_structure->title\"><!--[if !IE]>--><span class='float-ptr'>&gt;</span><!--<![endif]-->$nav_structure->title<!--[if lte IE 7]>&nbsp;&nbsp;&gt;<![endif]--> <!--[if (!IE)|(gt IE 7)]><span class='float-ptr'>&gt;</span><![endif]--></a>";
			$retStr .= "\n\t\t\t"."<ul>";

			foreach ($nav_structure->children as $child_page) {
				$target = $child_page->target ? " target='".$child_page->target."' " : ' ';
				$child_href = trim($child_page->link) == "" ? " href='#' onclick='return false;' " : " href='$child_page->link' ";
				//$retStr .= "\n\t\t\t\t"."<li><a href='$child_page->link' $target";
				$retStr .= "\n\t\t\t\t"."<li><a $child_href $target";
				$retStr .= suffusion_parse_nav_structure($child_page, $position);
			}
			$retStr .= "\n\t\t\t\t"."</ul>";
			$retStr .= "\n\t\t\t\t"."</li>";
		}
		else {
			$retStr .= " title=\"$nav_structure->title\">$nav_structure->title</a></li>";
		}
	}
	return $retStr;
}

function suf_get_excluded_categories($prefix) {
    global $$prefix;
	$inclusions = $$prefix;
    $inclusions_array = explode(',', $inclusions);

    if (is_array($inclusions_array)) {
        $translations_array = wpml_lang_object_ids($inclusions_array, 'category');
    }
    if (is_array($translations_array)) {
        foreach ($translations_array as $translation) {
            if (!in_array($translation, $inclusions_array)) {
                $inclusions_array[count($inclusions_array)] = $translation;
            }
        }
    }
    if (is_array($inclusions_array)) {
        $inclusions = implode(',', $inclusions_array);
    }

	$args = array(
        "type" => "post",
		"orderby" => "name",
		"exclude" => $inclusions,
	);

	$excluded = get_categories($args);
	return $excluded;
}

function suf_get_included_bookmarks($prefix, $all = false) {
    global $$prefix;
	$inclusions = $$prefix;
    if (function_exists('mylinkorder')) {
        $order_by = 'order';
    }
    else {
        $order_by = 'name';
    }
    $args = array(
        "order" => "ASC",
        "orderby" => $order_by,
    );
    $links = get_bookmarks($args);
	if ($all) {
		return $links;
	}
    $included = array();
    $inclusions_array = explode(',',$inclusions);
    if ($inclusions_array && count($inclusions_array) > 0) {
        foreach ($links as $bookmark) {
            if (in_array($bookmark->link_id, $inclusions_array)) {
                $included[count($included)] = $bookmark;
            }
        }
    }
	return $included;
}

function create_nav_for_all($position = 'main', $page_option = 'suf_nav_pages', $cats_option = 'suf_nav_cats', $links_option = 'suf_nav_links') {
	global $post, $suf_nav_pages_style, $suf_nav_page_tab_title, $suf_nav_page_tab_link, $suf_navt_pages_style, $suf_navt_page_tab_title, $suf_navt_page_tab_link;
	global $suf_nav_cat_style, $suf_nav_cat_tab_title, $suf_nav_cat_tab_link, $suf_navt_cat_style, $suf_navt_cat_tab_title, $suf_navt_cat_tab_link;
	global $suf_nav_links_style, $suf_nav_links_tab_title, $suf_nav_links_tab_link, $suf_navt_links_style, $suf_navt_links_tab_title, $suf_navt_links_tab_link;
	global $suf_nav_entity_order, $suf_navt_entity_order, $suf_nav_pages_all_sel, $suf_nav_cats_all_sel, $suf_nav_links_all_sel, $suf_navt_pages_all_sel, $suf_navt_cats_all_sel, $suf_navt_links_all_sel;

	switch ($position) {
		case 'top':
			$pages_style = $suf_navt_pages_style;
			$page_tab_title = stripslashes($suf_navt_page_tab_title);
			$page_tab_link = $suf_navt_page_tab_link;
			$page_all_sel = $suf_navt_pages_all_sel;
			$cat_style = $suf_navt_cat_style;
			$cat_tab_title = stripslashes($suf_navt_cat_tab_title);
			$cat_tab_link = $suf_navt_cat_tab_link;
			$cat_all_sel = $suf_navt_cats_all_sel;
			$links_style = $suf_navt_links_style;
			$links_tab_title = stripslashes($suf_navt_links_tab_title);
			$links_tab_link = $suf_navt_links_tab_link;
			$link_all_sel = $suf_navt_links_all_sel;
			$entity_order = $suf_navt_entity_order;
			break;
		default:
			$pages_style = $suf_nav_pages_style;
			$page_tab_title = stripslashes($suf_nav_page_tab_title);
			$page_tab_link = $suf_nav_page_tab_link;
			$page_all_sel = $suf_nav_pages_all_sel;
			$cat_style = $suf_nav_cat_style;
			$cat_tab_title = stripslashes($suf_nav_cat_tab_title);
			$cat_tab_link = $suf_nav_cat_tab_link;
			$cat_all_sel = $suf_nav_cats_all_sel;
			$links_style = $suf_nav_links_style;
			$links_tab_title = stripslashes($suf_nav_links_tab_title);
			$links_tab_link = $suf_nav_links_tab_link;
			$link_all_sel = $suf_nav_links_all_sel;
			$entity_order = $suf_nav_entity_order;
			break;
	}

	if ($page_all_sel == 'selected') {
		$pages_excluded = get_excluded_pages($page_option);
	}
	else {
		$pages_excluded = array();
	}

	if ($cat_all_sel == 'selected') {
		$categories_excluded = suf_get_excluded_categories($cats_option);
	}
	else {
		$categories_excluded = array();
	}

	if ($link_all_sel == 'selected') {
		$bookmarks_included = suf_get_included_bookmarks($links_option, false);
	}
	else {
		$bookmarks_included = suf_get_included_bookmarks($links_option, true);
	}

	$nav_structure = new Suf_Navigation_Structure(0, 'Root', '', 0);
	$nav_structure->children = array();
    $entity_order = suf_get_entity_order($entity_order, 'nav');
    $entity_order = explode(',', $entity_order);
    foreach ($entity_order as $entity) {
        if ($entity == 'pages') {
            if ($pages_style == 'rolled-up') {
                $page_structure = suffusion_get_page_structure(0, $page_tab_title, $page_tab_link, 1, $pages_excluded);
                if ($page_structure->children != null && count($page_structure->children) > 0) {
                    $nav_structure->children[count($nav_structure->children)] = $page_structure;
                }
            }
            else if ($pages_style == 'flattened') {
                $page_structure = suffusion_get_page_structure(0, $page_tab_title,  $page_tab_link, 0, $pages_excluded);
                if ($page_structure->children != null && count($page_structure->children) > 0) {
                    foreach ($page_structure->children as $child_structure) {
                        $nav_structure->children[count($nav_structure->children)] = $child_structure;
                    }
                }
            }
        }
        else if ($entity == 'categories') {
            if ($cat_style == 'rolled-up') {
                $category_structure = suffusion_get_category_structure(0, $cat_tab_title, $cat_tab_link, 1, $categories_excluded, 'page');
                if ($category_structure->children != null && count($category_structure->children) > 0) {
                    $nav_structure->children[count($nav_structure->children)] = $category_structure;
                }
            }
            else if ($cat_style == 'flattened') {
                $category_structure = suffusion_get_category_structure(0, $cat_tab_title, $cat_tab_link, 0, $categories_excluded, 'cat');
                foreach ($category_structure->children as $child_structure) {
                    $nav_structure->children[count($nav_structure->children)] = $child_structure;
                }
            }
        }
        else if ($entity == 'links') {
            if ($links_style == 'rolled-up') {
                $bookmark_structure = suffusion_get_bookmark_structure(0, $links_tab_title, $links_tab_link, 1, $bookmarks_included, 'page');
                if ($bookmark_structure->children != null && count($bookmark_structure->children) > 0) {
                    $nav_structure->children[count($nav_structure->children)] = $bookmark_structure;
                }
            }
            else if ($links_style == 'flattened') {
                $bookmark_structure = suffusion_get_bookmark_structure(0, $links_tab_title, $links_tab_link, 0, $bookmarks_included, 'link');
                foreach ($bookmark_structure->children as $child_structure) {
                    $nav_structure->children[count($nav_structure->children)] = $child_structure;
                }
            }
        }
    }

	$retStr = suffusion_parse_nav_structure($nav_structure, $position);
	return $retStr;
}

function create_nav($echo = true, $type = "pages", $position = 'main', $page_option = 'suf_nav_pages', $cats_option = 'suf_nav_cats', $links_option = 'suf_nav_links') {
	$retStr = "";
	if ($type == "pages") {
		$retStr = create_nav_for_all($position, $page_option, $cats_option, $links_option);
	}
	if ($echo) {
		echo $retStr;
	}
	return $retStr;
}

/**
 * If more than one page exists, return TRUE.
 */
function show_page_nav() {
	global $wp_query;
	return ($wp_query->max_num_pages > 1);
}
/**
 * If more than one post exists, return TRUE.
 */
function suffusion_post_count() {
	$post_count = wp_count_posts();
	return $post_count->publish;
}

function check_integer($val) {
    return ($val !== true) && ((string)abs((int) $val)) === ((string) ltrim($val, '-0'));
}

function get_size_from_field($val, $default) {
	$ret = $default;
	if (substr(trim($val), -2) == "px") {
		$test_str = trim(substr(trim($val), 0, strlen(trim($val)) - 2));
		if (check_integer($test_str)) {
			$ret = $test_str."px";
		}
	}
	else if (check_integer(trim($val))) {
		$ret = trim($val)."px";
	}
	return $ret;
}

function get_numeric_size_from_field($val, $default) {
	$ret = $default;
	if (substr(trim($val), -2) == "px") {
		$test_str = trim(substr(trim($val), 0, strlen(trim($val)) - 2));
		if (check_integer($test_str)) {
			$ret = (int)$test_str;
		}
	}
	else if (check_integer(trim($val))) {
		$ret = (int)(trim($val));
	}
	return $ret;
}

function suffusion_get_formatted_sbtab_array($prefix, $spawn = true) {
	global $sidebar_tabs, $spawned_options;

	foreach ($sidebar_tabs as $tab => $tab_options) {
		$spawned_options[count($spawned_options)] = array('id' => $prefix.'_'.$tab, 'type' => 'checkbox', 'parent' => $prefix, 'std' => 'false');
	}
	return $sidebar_tabs;
}

function suffusion_get_category_structure($id, $title, $link, $depth, $exclusion_list = array(), $type = 'cat') {
	$category_structure = new Suf_Navigation_Structure($id, $title, $link, $depth, $type);
	$category_structure->children = array();
	$exclude_categories = array();
	foreach ($exclusion_list as $category) {
		$exclude_categories[count($exclude_categories)] = $category->cat_ID;
	}
	$exclude = implode(',', $exclude_categories);
	$child_cats = get_categories(array('type' => 'post', 'child_of' => $id, 'parent' => $id, 'exclude' => $exclude));
    $child_cat_ids = array();
    if ($child_cats) {
        foreach ($child_cats as $child_cat) {
            $child_cat_ids[count($child_cat_ids)] = $child_cat->cat_ID;
        }
    }
    $lang_specific_cat_ids = wpml_lang_object_ids($child_cat_ids, 'category');
    if (count($lang_specific_cat_ids) != 0) {
        $include = implode(',',$lang_specific_cat_ids);
        if (function_exists('mycategoryorder')) {
            $order_by = 'order';
        }
        else {
            $order_by = 'name';
        }
        $child_cats = get_categories(array('type' => 'post', 'include' => $include, 'orderby' => $order_by));
        if ($child_cats == null) { $child_cats = array(); }
        $child_depth = $depth + 1;
        foreach ($child_cats as $child_cat) {
            $child_cat_link = get_category_link($child_cat->cat_ID);
            $child = suffusion_get_category_structure($child_cat->cat_ID, $child_cat->cat_name, $child_cat_link, $child_depth, $exclusion_list);
            $category_structure->children[count($category_structure->children)] = $child;
        }
    }
	return $category_structure;
}

function suffusion_get_category_ancestor_count($cat, $counter) {
	if ($cat == null || !$cat) {
		return $counter;
	}
	$count = $counter;
	if ($cat->category_parent) {
		$parent = get_category($cat->category_parent);
		$count = suffusion_get_category_ancestor_count($parent, $count + 1);
	}
	return $count;
}

function suffusion_get_bookmark_structure($id, $title, $link, $depth, $inclusion_list = array(), $type = 'link', $target = false) {
	$bookmark_structure = new Suf_Navigation_Structure($id, $title, $link, $depth, $type, $target);
	$bookmark_structure->children = array();
    $child_depth = $depth + 1;
    foreach ($inclusion_list as $bookmark) {
        $child = suffusion_get_bookmark_structure($bookmark->link_id, $bookmark->link_name, $bookmark->link_url, $child_depth, array(), 'link', $bookmark->link_target);
        $bookmark_structure->children[count($bookmark_structure->children)] = $child;
    }
	return $bookmark_structure;
}

function suf_get_allowed_pages($prefix) {
	global $suffusion_options;
	$allowed = array();
	$selected = $suffusion_options[$prefix];
    if (!empty($selected)) {
        $selected_pages = explode(',', $selected);
        if (is_array($selected_pages) && count($selected_pages) > 0) {
            foreach ($selected_pages as $page_id) {
                $page = get_page($page_id);
                $allowed[count($allowed)] = $page;
            }
        }
    }
	return $allowed;
}

function suf_get_formatted_link_array($prefix, $spawn = true) {
	global $spawned_options, $link_array;
	$ret = array();
	$args = array(
		"order" => "ASC",
		"orderby" => 'name',
	);
	$links = get_bookmarks($args);
	if ($links == null) { $links = array(); }
	foreach ($links as $link) {
		if ($link_array == null) {
			$ret[$link->link_id] = array("title" => $link->link_name);
		}
		if ($spawn) {
			$spawned_options[count($spawned_options)] = array(  "id" => $prefix."_".$link->link_id,
														        "type" => "checkbox",
														        "parent" => $prefix,
																"std" => "false");
		}
	}
	if ($link_array == null) {
		$link_array = $ret;
		return $link_array;
	}
	else {
		return $link_array;
	}
}

/* Functions for WPML compatibility */
function wpml_lang_object_id($id, $type){
    if(function_exists('icl_object_id')) {
        return icl_object_id($id, $type, true);
    } else {
        return $id;
    }
}

function wpml_lang_object_ids($ids_array, $type) {
    if(function_exists('icl_object_id')) {
        $res = array();
        foreach ($ids_array as $id) {
            $xlat = icl_object_id($id, $type, false);
            if (!is_null($xlat) && !in_array($xlat, $res)) {
                $res[] = $xlat;
            }
        }
        return $res;
    }
    else {
        return $ids_array;
    }
}
/* End functions for WPML compatibility */

function suffusion_tab_array_prepositions() {
    global $sidebar_tabs;
    $ret = array();
    foreach ($sidebar_tabs as $key => $value) {
        $ret[count($ret)] = array('key' => $key, 'value' => $value['title']);
    }
    return $ret;
}

function suffusion_nav_entity_prepositions() {
    $ret = array(array('key' => 'pages', 'value' => 'Pages'), array('key' => 'categories', 'value' => 'Categories'), array('key' => 'links', 'value' => 'Links'));
    return $ret;
}

function suf_get_formatted_options_array($prefix, $options_array) {
	global $spawned_options;
	$ret = array();
    foreach ($options_array as $option_key => $option_value) {
        $ret[$option_key] = array('title' => $option_value, 'depth' => 1);
        $spawned_options[] = array('id' => $prefix.'_'.$option_key, 'type' => 'checkbox', 'parent' => $prefix, 'std' => 'false');
    }
    return $ret;
}

function suf_get_entity_order($entity_order, $entity_type='nav') {
    $ret = array();
    if (is_array($entity_order)) {
        foreach ($entity_order as $element) {
            $ret[] = $element['key'];
        }
        $ret = implode(',',$ret);
    }
    else {
        $defaults = suffusion_entity_prepositions($entity_type);
        $ret = explode(',', $entity_order);
        $default_array = array();
        foreach ($defaults as $default) {
            $default_array[] = $default['key'];
            if (!in_array($default['key'], $ret)) {
                $ret[] = $default['key'];
            }
        }
        $ret_array = array();
        foreach ($ret as $ret_entry) {
            if (in_array($ret_entry, $default_array)) {
                $ret_array[] = $ret_entry;
            }
        }
        $ret = implode(',', $ret_array);
    }
    return $ret;
}

function suffusion_update_available($theme) {
    static $themes_update;
    if ( !isset($themes_update) )
        $themes_update = get_transient('update_themes');

    if ( is_object($theme) && isset($theme->stylesheet) )
        $stylesheet = $theme->stylesheet;
    elseif ( is_array($theme) && isset($theme['Stylesheet']) )
        $stylesheet = $theme['Stylesheet'];
    else
        return false; //No valid info passed.

    if (isset($themes_update->response[ $stylesheet ])) {
        return true;
    }
    return false;
}

function suffusion_get_full_content_count() {
	global $suffusion;
	$context = $suffusion->get_context();
	$full_post_count = 0;
	if (in_array('category', $context)) {
		$full_post_count = (int)($suffusion->option('suf_category_fc_number'));
	}
	else if (in_array('author', $context)) {
		$full_post_count = (int)($suffusion->option('suf_author_fc_number'));
	}
	else if (in_array('tag', $context)) {
		$full_post_count = (int)($suffusion->option('suf_tag_fc_number'));
	}
	else if (in_array('search', $context)) {
		$full_post_count = (int)($suffusion->option('suf_search_fc_number'));
	}
	else if (in_array('date', $context)) {
		$full_post_count = (int)($suffusion->option('suf_archive_fc_number'));
	}
	else if (in_array('home', $context) || in_array('blog', $context)) {
		$full_post_count = (int)($suffusion->option('suf_index_fc_number'));
	}
	else if (in_array('page', $context)) {
		if (in_array('posts.php', $context)) {
			$full_post_count = (int)($suffusion->option('suf_pop_fc_number'));
		}
	}
	return $full_post_count;
}

function suffusion_nr_entity_prepositions() {
    $ret = array(array('key' => 'current', 'value' => 'Currently Reading'), array('key' => 'unread', 'value' => 'Not Yet Read'), array('key' => 'completed', 'value' => 'Completed'));
    return $ret;
}

function suffusion_entity_prepositions($entity_type = 'nav') {
	if ($entity_type == 'nav') {
		$ret = array(array('key' => 'pages', 'value' => 'Pages'), array('key' => 'categories', 'value' => 'Categories'), array('key' => 'links', 'value' => 'Links'));
	}
	else if ($entity_type == 'nr') {
		$ret = array(array('key' => 'current', 'value' => 'Currently Reading'), array('key' => 'unread', 'value' => 'Not Yet Read'), array('key' => 'completed', 'value' => 'Completed'));
	}
    return $ret;
}

if (function_exists('add_theme_support')) {
	add_theme_support('post-thumbnails');
}

require_once (TEMPLATEPATH . "/suffusion.php");
include_once (TEMPLATEPATH . "/widget-areas.php");
include_once (TEMPLATEPATH . "/functions/actions.php");
include_once (TEMPLATEPATH . "/functions/filters.php");
include_once (TEMPLATEPATH . "/functions/shortcodes.php");

$custom_php_file = $suffusion_options['suf_custom_php_file'];
if ($custom_php_file !== FALSE) {
	$custom_php_file = stripslashes($custom_php_file);
	if (substr($custom_php_file, 0, 1) == "/") {
		$custom_php_file = substr($custom_php_file, 1);
	}
	if (trim($custom_php_file) != "" && file_exists(WP_CONTENT_DIR."/".$custom_php_file)) {
		include_once(WP_CONTENT_DIR."/".$custom_php_file);
	}
}

$suffusion = new Suffusion();
$suffusion->init();
?>